/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include "TestControllerPool.hpp"

// not to derive: just to get default parameters
#include "../ConnectionSetup.hpp"

#include "ControllerPool.hpp"
#include "Statement.hpp"
#include "Connection.hpp"
#include "ConnectionParameters.hpp"

#include "Common.hpp"

#include <string>
#include <iostream>

using std::wstring;

using namespace CarobNS;

void TestControllerPool::setUp()
{
}

void TestControllerPool::tearDown()
{
}

void TestControllerPool::testRoundRobinSequence()
{
  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST1, 4000);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST1, 4001);
  ControllerInfo c3(ConnectionSetup::DEFAULT_HOST1, 4002);
  ControllerInfo c4(ConnectionSetup::DEFAULT_HOST1, 4003);
  
  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);
  v.push_back(c3);
  v.push_back(c4);

  RoundRobinControllerPool cp(v, ConnectionSetup::DEFAULT_PING_DELAY, ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT);
  CPPUNIT_ASSERT(cp.getController() == c1);
  cp.forceControllerDown(c2);
  cp.forceControllerDown(c4);
  CPPUNIT_ASSERT(cp.getController() == c3);
  CPPUNIT_ASSERT(cp.getController() == c1);
  
  v.clear();
}
void getPoolAndAssertSequence(bool firstCtrlFirst)
{
  // Declaring controllers in reverse order on purpose
  // this will avoid reusing pool from previous tests
  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST2, ConnectionSetup::DEFAULT_PORT2);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1);
  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);

  // Get a new pool
  ConnectionParameters prms (v, ConnectionSetup::DEFAULT_DB,
                                ConnectionSetup::DEFAULT_USER,
                                ConnectionSetup::DEFAULT_PASSWD);
  AbstractControllerPool& pool = prms.getControllerPool();
  // Test sequence
  if (firstCtrlFirst)
  {
    CPPUNIT_ASSERT(pool.getController() == c1);
    CPPUNIT_ASSERT(pool.getController() == c2);
    CPPUNIT_ASSERT(pool.getController() == c1);
  }
  else
  {
    CPPUNIT_ASSERT(pool.getController() == c2);
    CPPUNIT_ASSERT(pool.getController() == c1);
    CPPUNIT_ASSERT(pool.getController() == c2);
  }
}

void TestControllerPool::testSequenceWithDelete()
{
  getPoolAndAssertSequence(true);
  // In the next call, the pools should be the same.
  // Check this by asserting that the next ctrl given will be the 2nd one
  getPoolAndAssertSequence(false);
  // do another round just to have the second controller as the next one to be
  // given by the pool
  getPoolAndAssertSequence(true);

  // Wait 10s, time after wich the pool will be discarded
  sleep(10);

  // retrieve a fake pool so pools status are updated
  std::vector<ControllerInfo> vFake;
  vFake.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1));
  ConnectionParameters cpFake(vFake, ConnectionSetup::DEFAULT_DB,
                                     ConnectionSetup::DEFAULT_USER,
                                     ConnectionSetup::DEFAULT_PASSWD);
  cpFake.getControllerPool();

  // In this call, the pool should be a new one, the sequence should start
  // with the first controller first
  getPoolAndAssertSequence(true);
}

void TestControllerPool::testNoMoreController()
{
  const wstring fctName(L"TestControllerPool::testNoMoreController");

  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST1, 4000);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST1, 4001);
  ControllerInfo c3(ConnectionSetup::DEFAULT_HOST1, 4002);
  ControllerInfo c4(ConnectionSetup::DEFAULT_HOST1, 4003);

  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);
  v.push_back(c3);
  v.push_back(c4);

  RoundRobinControllerPool cp(v, ConnectionSetup::DEFAULT_PING_DELAY, ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT);
  cp.forceControllerDown(c1);
  cp.forceControllerDown(c2);
  cp.forceControllerDown(c3);
  cp.forceControllerDown(c4);
  
  try
  {
    if (isInfoEnabled())
      logInfo(fctName, L"Trying to get controller - should fail");
    cp.getController();
    CPPUNIT_ASSERT(false); //should have thrown an exception
  }
  catch (NoMoreControllerException nmce)
  {
    if (isErrorEnabled())
      logError(fctName, L"getController() failed (this is ok). Exception: "
          + nmce.description());
  }

  v.clear();
}

void TestControllerPool::testPoolManager()
{
  const wstring fctName(L"TestControllerPool::testPoolManager");

  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST1, 4000);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST1, 4001);
  ControllerInfo c3(ConnectionSetup::DEFAULT_HOST1, 4002);
  ControllerInfo c4(ConnectionSetup::DEFAULT_HOST1, 4003);

  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);
  v.push_back(c3);
  v.push_back(c4);

  // test 2 identical pools
  AbstractControllerPool* cp1 = &(ControllerPoolManager::getPool(v, ROUND_ROBIN,
      ConnectionSetup::DEFAULT_PING_DELAY, ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT));
  AbstractControllerPool* cp2 = &(ControllerPoolManager::getPool(v, ROUND_ROBIN,
      ConnectionSetup::DEFAULT_PING_DELAY, ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT));
  CPPUNIT_ASSERT(cp1 == cp2);

  // test with different controllers
  std::vector<ControllerInfo> v2;
  v2.push_back(c1);
  AbstractControllerPool* cp3 = &(ControllerPoolManager::getPool(v2, ROUND_ROBIN,
      ConnectionSetup::DEFAULT_PING_DELAY, ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT));
  CPPUNIT_ASSERT(cp3 != cp2);

  // test with different ping delay
  AbstractControllerPool* cp4 = &(ControllerPoolManager::getPool(v, ROUND_ROBIN,
      10, ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT));
  CPPUNIT_ASSERT(cp4 != cp2);

  // test with different controller timeout
  AbstractControllerPool* cp5 = &(ControllerPoolManager::getPool(v, ROUND_ROBIN,
      ConnectionSetup::DEFAULT_PING_DELAY, 10));
  CPPUNIT_ASSERT(cp5 != cp2);

  v.clear();
}


void TestControllerPool::testSuspectListUpdate()
{
  const wstring fctName(L"TestControllerPool::testSuspectListUpdate");
  std::wcerr<<L"Launch 4 servers on localhost:4000-4003 and press <return>"<<std::endl;
  std::cin.ignore(std::numeric_limits<std::streamsize>::max(), '\n');
  
  ControllerInfo c1(ConnectionSetup::DEFAULT_HOST1, 4000);
  ControllerInfo c2(ConnectionSetup::DEFAULT_HOST1, 4001);
  ControllerInfo c3(ConnectionSetup::DEFAULT_HOST1, 4002);
  ControllerInfo c4(ConnectionSetup::DEFAULT_HOST1, 4003);
  
  std::vector<ControllerInfo> v;
  v.push_back(c1);
  v.push_back(c2);
  v.push_back(c3);
  v.push_back(c4);

  RoundRobinControllerPool cp(v, ConnectionSetup::DEFAULT_PING_DELAY, ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT);
  cp.forceControllerDown(c1);
  cp.forceControllerDown(c2);
  CPPUNIT_ASSERT(cp.getController() == c3);
  CPPUNIT_ASSERT(cp.getController() == c4);
  CPPUNIT_ASSERT(cp.getController() == c1);
  CPPUNIT_ASSERT(cp.getController() == c2);

  v.clear();
}

CppUnit::Test* TestControllerPool::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite("10.TestControllerPool");
  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "10.TestControllerPool::testRoundRobinSequence",
                                 &TestControllerPool::testRoundRobinSequence));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "10.TestControllerPool::testSequenceWithDelete",
                                 &TestControllerPool::testSequenceWithDelete));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "10.TestControllerPool::testNoMoreController",
                                 &TestControllerPool::testNoMoreController));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "10.TestControllerPool::testPoolManager",
                                 &TestControllerPool::testPoolManager));
//Test disabled, needs user interaction
/*  suiteOfTests->addTest(new CppUnit::TestCaller<TestControllerPool>(
                                 "10.TestControllerPool::testSuspectListUpdate",
                                 &TestControllerPool::testSuspectListUpdate));
*/
  return suiteOfTests;
}
