/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include <cppunit/ui/text/TestRunner.h>
// For QT ui, replace the above line by this:
//#include <cppunit/ui/qt/TestRunner.h>
//#include <qt3/qapplication.h>

#include <cppunit/TestResult.h>
#include <cppunit/BriefTestProgressListener.h>
#include <cppunit/TextTestProgressListener.h>

#include "CarobException.hpp"
#include "StringCodecs.hpp"
#include "Common.hpp"
#ifdef CAROB_LOG4CXX_NAME
#include <log4cxx/logger.h>
#include <log4cxx/basicconfigurator.h>
#include <log4cxx/propertyconfigurator.h>
#endif

#include "CarobProtector.hpp"
#include "ConnectionSetup.hpp"
#include "ControllerInfo.hpp"
#include "Connection.hpp"
#include "Statement.hpp"
#include "DriverResultSet.hpp"

#include "01-Unit/TestStringCodecs.hpp"
#include "10-Connection/TestConnect.hpp"
#include "10-Connection/TestControllerPool.hpp"
#include "10-Connection/TestFailOver.hpp"
#include "10-Connection/TestBeginCommitRollback.hpp"
#include "20-Write/TestExecWriteRequest.hpp"
#include "30-ResultSet/TestSimpleUnicode.hpp"
#include "30-ResultSet/TestExecReadRequest.hpp"
#include "30-ResultSet/TestDriverResultSet.hpp"
#include "30-ResultSet/TestBigDecimal.hpp"
#include "35-ResultList/TestExec.hpp"
#include "40-Parameter-PreparedStatement/TestParameterStatement.hpp"
#include "40-Parameter-PreparedStatement/TestPreparedStatement.hpp"
#include "40-Parameter-PreparedStatement/TestIEEE754.hpp"

#include <string>
#include <iostream>
#include <fstream>

using namespace CarobNS;
using namespace std;

const string TEST_TABLES_SCRIPT = "testTables.sql";

bool needToCreateTestTables()
{
  vector<ControllerInfo> ctrls;
  Connection* connectionPtr = NULL;
  try
  {
    ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1));
    ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST2, ConnectionSetup::DEFAULT_PORT2));
    ConnectionParameters connectionPrms(ctrls,
                                        ConnectionSetup::DEFAULT_DB,
                                        ConnectionSetup::DEFAULT_USER,
                                        ConnectionSetup::DEFAULT_PASSWD,
                                        ConnectionSetup::DEFAULT_POLICY,
                                        ConnectionSetup::DEFAULT_PING_DELAY,
                                        ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT,
                                        ConnectionSetup::DEFAULT_CONNECTION_PERSISTENCY,
                                        ConnectionSetup::DEFAULT_RETRIEVE_SQL_WARNINGS);
    connectionPtr = new Connection(connectionPrms);
  }
  catch (CarobException e)
  {
    logError(L"needToCreateTestTables()", L"Connection failed: " + e.description()); 
    return false;
  }
  bool needToCreate = true;
  try
  {
    Statement* statementPtr = connectionPtr->createStatement();
    DriverResultSet* drsPtr = statementPtr->executeQuery(L"SELECT * FROM product");
    if (drsPtr->next())
    {
      drsPtr = statementPtr->executeQuery(L"SELECT * FROM address");
      if (drsPtr->next())
        needToCreate = false;
    }
  }
  catch (CarobException e)
  {
    logError(L"needToCreateTestTables()", L"Script execution failed: " + e.description()); 
  }
  delete connectionPtr;
  return needToCreate;
}

bool createTestTables(string testTablesScript)
{
  ifstream script(testTablesScript.c_str(), ios::in);
  if (!script)
  {
    cout<<"Can't open "<<testTablesScript<<endl;
    return false;
  }
  vector<ControllerInfo> ctrls;
  Connection* connectionPtr = NULL;
  try
  {
    ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, ConnectionSetup::DEFAULT_PORT1));
    ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST2, ConnectionSetup::DEFAULT_PORT2));
    ConnectionParameters connectionPrms(ctrls,
                                        ConnectionSetup::DEFAULT_DB,
                                        ConnectionSetup::DEFAULT_USER,
                                        ConnectionSetup::DEFAULT_PASSWD,
                                        ConnectionSetup::DEFAULT_POLICY,
                                        ConnectionSetup::DEFAULT_PING_DELAY,
                                        ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT,
                                        ConnectionSetup::DEFAULT_CONNECTION_PERSISTENCY,
                                        ConnectionSetup::DEFAULT_RETRIEVE_SQL_WARNINGS);
    connectionPtr = new Connection(connectionPrms);
  }
  catch (CarobException e)
  {
    logError(L"createTestTables()", L"Connection failed: " + e.description()); 
    return false;
  }

  dropTableIfExists(connectionPtr, L"product");
  dropTableIfExists(connectionPtr, L"address");

  try
  {
    Statement* statementPtr = connectionPtr->createStatement();

    string sql;
    getline(script, sql, '\n');
    while (sql.length() > 1)
    {
      statementPtr->executeUpdate(fromString(sql));
      getline(script, sql, '\n');
    }
  }
  catch (CarobException e)
  {
    logError(L"createTestTables()", L"Script execution failed: " + e.description()); 
    return false;
  }
  delete connectionPtr;
  return true;
}

int main (int argc, char **argv)
{
#ifdef CAROB_LOG4CXX_NAME
  // Set up a simple configuration that logs on the console.
  log4cxx::BasicConfigurator::configure();
#endif

	// Log Level:
  // available levels are found in log4cxx documentation
  // or in file include/Common.hpp
  setLogLevel(LOG_LEVEL_WARN);

#ifdef __MINGW32__
  //init windows socket lib
  WSADATA WSAData;
  if (isInfoEnabled())
    logInfo(L"Carob unit tests", L"Initializing windows socket library");
  if (WSAStartup(MAKEWORD(2,0), &WSAData) != 0)
  {
    if (isErrorEnabled())
      logError(L"read_example", L"Unable to find suitable windows socket library");
    throw ConnectionException(L"Unable to find suitable windows socket library");
    return false;
  }
#endif

  // See CAROB-56 and related
  // std::set_unexpected(UnexpectedException::convertUnexpected);

  CppUnit::TextUi::TestRunner runner;
  // For QT ui, replace the above line by this:
  //QApplication app(argc, argv);
  //CppUnit::QtUi::TestRunner runner;
  runner.addTest(TestConnect::suite());
  runner.addTest(TestExecWriteRequest::suite());
  runner.addTest(TestExecReadRequest::suite());
  runner.addTest(TestExec::suite());
  runner.addTest(TestBeginCommitRollback::suite());
  runner.addTest(TestBigDecimal::suite());
  runner.addTest(TestDriverResultSet::suite());
  runner.addTest(TestStringCodecs::suite());
  runner.addTest(TestSimpleUnicode::suite());
  runner.addTest(TestParameterStatement::suite());
  runner.addTest(TestIEEE754::suite());
  runner.addTest(TestControllerPool::suite());
  runner.addTest(TestFailOver::suite());

  // add our own protector
  CarobProtector* cprot = new CarobProtector();
  runner.eventManager().pushProtector(cprot);

#if 0
  // Add a minimal listener that print dots as test run.
  CppUnit::TextTestProgressListener progress;
  runner.eventManager().addListener( &progress );
#else
  // this one prints test names
  CppUnit::BriefTestProgressListener listener;
  runner.eventManager().addListener( &listener );
#endif

  // workaround for CAROB-76
  std::cerr << std::endl;

  string testTablesScript = TEST_TABLES_SCRIPT;
  if (argc == 2)
    testTablesScript = argv[1];
  // Create needed tables for tests
  if (needToCreateTestTables())
  {
    cout<<"Creating test tables..."<<endl;
    if (!createTestTables(testTablesScript))
    {
      cerr<<"Failed to create test tables!"<<endl;
      return 1;
    }
  }
  else
    cout<<"No need to create tables, running tests..."<<endl;

  // to get debug info:
  setLogLevel(LOG_LEVEL_FATAL);
  // To run only one test:
  // runner.run("10.TestConnect::testConnectGood");
  // To run only one test class:
  // runner.run("10.TestConnect");
  runner.run();

  //Do *NOT* delete CarobProtector, it is done by the ProtectorChain (in pop())
#ifdef __MINGW32__
  WSACleanup();
#endif
  return 0;

}

